;;; virtual windows for the environment that go out through
;;; Tcl/Tk to display i.e. a visible virtual
;;;
;;; Inherits much of the functionality from the rpm-virtual-window
;;; but needs to send the commands to Tcl/Tk as well

(defvar *use-environment-window* 
  #+:ACTR-ENVIRONMENT t
  #-:ACTR-ENVIRONMENT nil)

;;; First define all the stuff necessary for it and it's possible 
;;; 'subviews' as it relates to the device interface

(defclass environment-exp-window (rpm-virtual-window)
  ())

(defmethod initialize-instance :after ((win environment-exp-window) &key)
  (send-env-window-update (list 'open (window-title win) (x-pos win)
                                (y-pos win) (width win) (height win))))

(defclass env-text-vdi (static-text-vdi)
  ()
  (:default-initargs
      :id (new-name-fct "TEXT")))

(defclass env-button-vdi (button-vdi)
  ()
  (:default-initargs
      :id (new-name-fct "BUTTON")))

(defclass env-line-vdi (v-liner)
  ()
  (:default-initargs
      :id (new-name-fct "LINE")))


(defmethod vv-click-event-handler ((btn env-button-vdi) where)
  (declare (ignore where))
  (when (functionp (action-function btn))
    (funcall (action-function btn) btn))
  (when *actr-enabled-p* 
    (send-env-window-update (list 'click (id btn)))))


(defmethod device-move-cursor-to ((vw environment-exp-window) (loc vector))
  (setf (cursor-pos vw) loc)
  (when (with-cursor-p (device-interface *mp*))
    (pm-proc-display))
  (send-env-window-update (list 'cursor (px loc) (py loc))))

(defmethod device-update-attended-loc ((wind environment-exp-window) xyloc)
  (send-env-window-update (list 'attention (px xyloc) (py xyloc))))



;;; Then define the UWI functions - which get called by the exp interface

(defmethod close-rpm-window ((win environment-exp-window))
  (setf (window-open? win) nil)
  (send-env-window-update (list 'close)))


(defmethod select-rpm-window ((win environment-exp-window))
  (send-env-window-update (list 'select)))

(defmethod add-visual-items-to-rpm-window ((win environment-exp-window) 
                                           &rest items)
  (dolist (item items)
    (add-subviews win item)
    (send-env-window-update 
     (case (type-of item)
       (env-button-vdi
        (list 'button (id item) (x-pos item) (y-pos item) 
          (width item) (height item) (dialog-item-text item)))
       (env-text-vdi
        (list 'text 
          (id item) (x-pos item) (y-pos item) 
          (color item)  (dialog-item-text item)))
       (env-line-vdi
        (list 'line
              (id item) (x-pos item) (y-pos item) 
              (color item) (width item) (height item)))))))




;;; virtual

(defmethod remove-visual-items-from-rpm-window ((win environment-exp-window) 
                                                &rest items)
  (dolist (item items)
    (remove-subviews win item)
    (send-env-window-update (list 'remove (id item)))))



;;; virtual

(defmethod remove-all-items-from-rpm-window ((win environment-exp-window))
  (apply #'remove-subviews win (subviews win))
  (send-env-window-update (list 'clear)))



;;; virtual

(defmethod make-button-for-rpm-window ((win environment-exp-window) 
                                       &key (x 0) (y 0) (text "Ok")  
                                       (action nil) (height 18) (width 60))
  (make-instance 'env-button-vdi
    :x-pos x 
    :y-pos y
    :dialog-item-text text
    :action action
    :height height
    :width width))


;;; virtual

(defmethod make-static-text-for-rpm-window ((win environment-exp-window) 
                                            &key (x 0) (y 0) (text "") 
                                            (height 20) (width 80))
  (make-instance 'env-text-vdi
    :x-pos x 
    :y-pos y
    :dialog-item-text text
    :height height
    :width width
    ))

(defmethod rpm-window-visible-status ((win environment-exp-window))
  t)

(defmethod make-line-for-rpm-window ((wind environment-exp-window) 
                                     start-pt end-pt &optional (color 'black))
  (make-instance 'env-line-vdi
    :color color
    :x-pos (first start-pt)
    :y-pos (second start-pt)
    :width (first end-pt)
    :height (second end-pt)))

;;; This is a little tricky, because if it's a real window in an IDE'ed
;;; Lisp, it's still got to do the right thing to prevent hanging, 
;;; which is lisp specific...

(defmethod allow-event-manager ((win environment-exp-window))
  #+:allegro-ide (process-pending-events)
  #+(and :mcl (not :openmcl)) (event-dispatch)
  )


;;; This function relies on the handlers from the environment existing...

(defun send-env-window-update (cmd)
  (dolist (x *env-windows*)
    (setf (update-form x) #'(lambda (x) cmd))
    (update-handler x nil)))