;;;  -*- mode: LISP; Package: CL-USER; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Mike Byrne
;;; Copyright   : (c)1997-2000 CMU/Rice U./Mike Byrne, All Rights Reserved
;;; Availability: public domain
;;; Address     : Rice University
;;;             : Psychology Department
;;;             : Houston,TX 77251-1892
;;;             : byrne@acm.org
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : speech-module.lisp
;;; Version     : 2.1b7
;;; 
;;; Description : Source code for the ACT-R/PM Speech Module.  This Module
;;;             : is pretty brain-damaged but should get the job done
;;;             : for short, simple strings.
;;; 
;;; Bugs        : 
;;; 
;;; Todo        : 
;;; 
;;; ----- History -----
;;; 97.04.02 Mike Byrne
;;;             : Incept date.
;;; 97.05.19 Mike Byrne
;;;             : Several minor bugs fixed, just clean-up stuff.
;;; 98.01.15 Mike Byrne
;;;             : Switched to use of OUTPUT-COMMAND.
;;; 98.01.19 Mike Byrne
;;;             : Added necessary RESET-MODULE command.
;;; 98.01.21 Mike Byrne
;;;             : Switched to use of QUEUE-EVENT command.
;;; 98.06.01 Mike Byrne
;;;             : Made relevant changes for b3.
;;; 99.04.06 mdb
;;;             : Tweaked SPEAK to send events to the device.
;;; 00.06.09 mdb
;;;             : Made SPEAK a regular movement style, which cleaned up the
;;;             : code considerably.
;;; 2002.05.23 mdb [b6]
;;;             : Added subvocalize stuff.
;;; 2002.06.24 mdb [b7]
;;;             : Subvocalize now makes the produced sound "internal".
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *char-per-syllable 3)


(defclass speech-module (pm-module)
  ((syllable-rate :accessor s-rate :initform 0.150)
   )
  (:default-initargs
    :version-string "2.1"
    :name :SPEECH))


(defmethod initialize-instance :after ((spch-mod speech-module) &key)
  (setf (state-dmo spch-mod)
        (make-dme 'speech-state 'module-state
                  '(module :speech modality free processor free preparation free
                    execution free)
                  :where :external)))


;;;; ---------------------------------------------------------------------- ;;;;
;;;; SPEAK movement style


(defstyle speak () text)

(defmethod num-to-prepare ((mvmt speak))
  3)

(defmethod compute-exec-time ((spch-mod speech-module) (mvmt speak))
  (init-time spch-mod))

(defmethod compute-finish-time ((spch-mod speech-module) (mvmt speak))
  (+ (exec-time mvmt)
     (* (s-rate spch-mod) (/ (length (text mvmt)) *char-per-syllable))))


(defmethod feat-differences ((s1 speak) (s2 speak))
  (if (string= (text s1) (text s2))
    0
    2))


(defmethod queue-output-events ((spch-mod speech-module) (mvmt speak))
  (new-word-sound (text mvmt) (+ (pm-time) (exec-time mvmt)))
  (queue-command
   :time (exec-time mvmt) :where :DEVICE :command 'output-speech
   :params (text mvmt)))



;;;; SUBVOCALIZE movement style

(defstyle subvocalize speak text)

(defmethod queue-output-events ((spch-mod speech-module) 
                                    (mvmt subvocalize))
  (new-word-sound (text mvmt) (+ (pm-time) (exec-time mvmt)) 'INTERNAL))
